; ******************************************************************************
;		(c) Mark Leyer 2010
; ******************************************************************************
;
;
;
;
;	Kurzanleitung:
;		Um einen Klang zu erzeugen, mssen die entsprechenden Frequenzen im SRAM eingestellt werden,
;		ebenso die Parameter fr die Wellenform, ggf. Pulsweite bei Rechteckwelle sowie die
;		Parameter fr die Hllkurve(n). Kanal 3 ist nur fr die Synchronisation oder Ringmodulation
;		mit Kanal 2, Wellenformwerte 0 bis 4 aktiviert die Ringmodulation mit der entsprechenden
;		Wellenform, Wert 5 aktiviert die Synchronisation (Wellenform egal), Wert 6 bedeutet, dass Kanal 2
;		nicht beeinflusst wird.
;
;		Sind alle Einstellungen vorgenommen, beginnt bei Setzen des entsprechenden Bits im Register
;		GateAkt die Hllkurve und verweilt nach durchlaufen der Attac- und Decay-Phasen solange auf
;		Sustain-Level, bis das Gate-Bit wieder gelscht wird, dann wird die Release-Phase ausgelst.
;
;
;	Einstellungen im SRAM:
;		Kanal1_Frequenz_H		Frequenz fr Kanal 1, (Wert = F * 28,8435456), High-Byte
;		Kanal1_Frequenz_L		w.v. Low-Byte
;		Kanal2_Frequenz_H		Frequenz fr Kanal 2, w.v.
;		Kanal2_Frequenz_L
;		Kanal3_Frequenz_H		Frequenz fr Kanal 3, w.v.
;		Kanal3_Frequenz_L
;		Kanal1_Wellenform		0=Rechteck, 1=Saegezahn, 2=Dreieck
;		Kanal2_Wellenform		   3=Rauschen, 4=Sinus
;		Kanal3_Wellenform		w.v. aktiviert Ringmodulation, 5=Sync, 6=off
;		Kanal1_Pulsweite		PWM-Wert fr die Rechteckschwingung Kanal 1
;		Kanal2_Pulsweite		w.v. Kanal 2
;		Kanal3_Pulsweite		w.v. Kanal 3
;		Kanal1_Attac			Attac-Zeit-Wert fr Kanal 1
;		Kanal1_Decay			Decay-Zeit-Wert fr Kanal 1
;		Kanal1_Sustain			Sustain-Level fr Kanal 1
;		Kanal1 Release			Release-Zeit-Wert fr Kanal 1
;		Kanal2_Attac			Attac-Zeit-Wert fr Kanal 2
;		Kanal2_Decay			Decay-Zeit-Wert fr Kanal 2
;		Kanal2_Sustain			Sustain-Level fr Kanal 2
;		Kanal2 Release			Release-Zeit-Wert fr Kanal 2
;
;	Zeiten fr Attac, Decay, Release: 0=6,55ms, 1=26ms, 2=52ms...256=26sec
;		Werte ber 0x0F hren sich aber 'Stufig' an
;
;		Register GateAkt:		0bAB0000CD
;								A - 1 fr weichen bergang zwischen zwei Tnen Kanal 2
;									bei 0 beginnt die Hllkurve auch jeweils mit Wert 0
;								B - w.v. Kanal 1
;								C - Gate fr Kanal 2
;								D - Gate fr Kanal 1
;
;	Einschrnkungen fr das Hauptprogramm:
;		Verwendbar sind die folgenden Register:
;			r0, r1, r2
;			r3  (= mult_R), gesichert ber Stack
;			r4  (= mult_2), gesichert ber Stack
;			r5  (= mult_1), gesichert ber Stack
;			r21 (= tempMain), gesichert ber Stack
;			r22 (= tempMain2), gesichert ber Stack
;			r23, r24, r25, X, Y
;			Z   (=ZL, ZH), gesichert ber Stack
;
;
;
;	Anmerkungen:
;		Aus (Takt-)Zeitgrnden habe ich grtenteils auf die Verwendung von RCALLs verzichtet
;		Daher ist der Code fr jeden Kanal einmal vorhanden. Die Kanle 1 und 2 sind auer
;		den Registern und SRAM-Speicherstellen soweit identisch
;
;





; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Unterschiedliche Versionen fr verschiedene Prozessoren
;	es sollte nur eine unkommentiert sein
;
;	Die Tiny85(45/25)-Version nutzt einen 8-bit-Timer und reduziert dafr
;		die Auflsung der Kanle
;	Die Tiny2313-Version nutzt einen 9-bit-Timer und gibt die volle
;		Auflsung aus, sonst gibt's keine besonderen Unterschiede
;	Die Tiny25/45-Varianten sollten mangels Unterschieden zum Tiny85
;		eigentlich laufen, kann's aber nicht real testen
;
;#define m_Tiny85
#define m_Tiny2313
;#define m_Tiny25		; nicht getestet
;#define m_Tiny45		; nicht getestet
;
;
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
; Es besteht die Mglichkeit, um Speicherplatz zu sparen, den Rauschgenerator
;	zu vereinfachen, spart ca. 210 Bytes FlashROM, das Rosa-Rauschen ist dann
;	halt noch etwas mehr Rosa
;
;#define m_Noise_Short
;
; +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++





#if defined(m_Tiny85)
.include "tn85def.inc"
#elif defined(m_Tiny2313)
.include "tn2313def.inc"
#elif defined(m_Tiny25)
.include "tn25def.inc"
#elif defined(m_Tiny45)
.include "tn45def.inc"
#else
#error "Kein Prozessor ausgewhlt"
#endif


.def temp			= r16
.def temp2			= r17
.def temp3			= r18
.def zero			= r15
.def CounterTemp1	= r14
.def CounterTemp2	= r13
.def CounterTemp3	= r12

.def T1DauerL		= r11
.def T1DauerH		= r10
.def T2DauerL		= r9
.def T2DauerH		= r8
.def GateAkt		= r7
.def GateAlt		= r6
.def mult_1			= r5
.def mult_2			= r4
.def mult_R			= r3

.def T1DauerM		= r19
.def T2DauerM		= r20


.def tempMain		= r21
.def tempMain2		= r22


; register frei: r0, r1, r2, r23, r24, r25, X, Y






.org 0x00
			rjmp reset

.org OVF1addr


; ************************************************************************

			; Register sichern, damit sie im Hauptprogramm weiter zur Verfgung stehen
			in temp, SREG
			push temp
			push ZL
			push ZH
			push mult_R
			push mult_1
			push mult_2



; *******************************************************************
;	grundlegender Aufbau:
;		zuerst kommen die Frequenzgeneratoren fr Kanal 1, dann Kanal 3 und Kanal 2
;		Kanal 3 wird nur verwendet, wenn sync oder ringmod fr Kanal 2 aktiviert sind
;		Danach Huellkurvengeneratoren fr Kanle 1+2
;		Danach Mischer fr Kanal 1+2, sowie Ausgabe ber PWM auf OCR1A
;


	; Frequenzgenerator Kanal 1



			; notwendige Werte aus dem SRAM holen zur Bearbeitung
			lds temp, Kanal1_Frequenz_L
			lds temp2, Kanal1_Frequenz_H
			lds CounterTemp1, K1_CL
			lds CounterTemp2, K1_CM
			lds CounterTemp3, K1_CH

			; Frequenzwerte des aktuellen Tons addieren mit 24-bit-Auflsung
			; es werden nur 20 bit verwendet
			add CounterTemp1, temp
			adc CounterTemp2, temp2
			adc CounterTemp3, zero
			brhc K1_FreqGen_1
			clr CounterTemp3

	K1_FreqGen_1:
			; die neuen Werte wieder im SRAM sichern
			sts K1_CH, CounterTemp3
			sts K1_CM, CounterTemp2
			sts K1_CL, CounterTemp1

			; die oberen 8 (von den 20) bit weiterverwenden
			mov temp, CounterTemp2
			mov temp2, CounterTemp3
			swap temp
			andi temp, 0b00001111
			swap temp2
			or temp, temp2

			; verzweigen nach Wellenform
			ldi ZL, low (K1_freq_Sprungtab)
			ldi ZH, high(K1_freq_Sprungtab)
			lds temp2, Kanal1_Wellenform
			add ZL, temp2
			adc ZH, zero
			ijmp

	K1_freq_Sprungtab:
			rjmp Kanal1_Rechteck
			rjmp K1_freq_X				; bei Dreieck muss nix berechnet werden
			rjmp Kanal1_Dreieck
			rjmp Kanal1_Rauschen
			rjmp Kanal1_Sinus


	Kanal1_Rechteck:
			; bei Rechteck, unterhalb Pulsweite null, sonst 0xFF
			lds temp2, Kanal1_Pulsweite
			cp temp, temp2
			brsh K1_freq_1
			clr temp
			rjmp K1_freq_X
		K1_freq_1:
			ser temp
			rjmp K1_freq_X

	Kanal1_Dreieck:
			; bei Dreieck erste Hlfte rauf auf 0xFF, zweite Hlfte wieder runter
			lsl temp				; *2
			brcs K1_freq_2
			rjmp K1_freq_X
		K1_freq_2:
			com temp				; und abwrts
			rjmp K1_freq_X

	Kanal1_Rauschen:
			; fr den Rauschgenerator
			; 'Zufalls'-zahl aus der ROM-Tabelle
			; plus weiteren 'Zufall' damit noch was 'zuflliger' wird
			ldi ZL, low (rauschen8bit*2)
			ldi ZH, high(rauschen8bit*2)
			swap CounterTemp1
			add temp, CounterTemp1
#if defined(m_Noise_Short)
			rjmp K1_sinus1
#else
			add ZL, temp
			adc ZH, zero
			lpm temp, Z
			rjmp K1_freq_X
#endif

	Kanal1_Sinus:
			; Zum Platzsparen gibt's nur das erste Viertel der Sinuskurve um Speicher
			; die anderen werden durch 'spiegeln' errechnet
			; zu speichern
			ldi ZL, low (sin8bit*2)
			ldi ZH, high(sin8bit*2)

	K1_sinus1:
			mov temp2, temp
			andi temp, 0b00111111

			sbrs temp2, 6			; zweites oder viertes Viertel des Sinus??
			rjmp K1_sinus2
	
			ldi temp3, 0x3F			; dann vertikal 'spiegeln'
			sub temp3, temp
			mov temp, temp3

	K1_sinus2:
			add ZL, temp
			adc ZH, zero

			lpm temp, Z

			sbrs temp2, 7			; erste oder zweite Hlfte des Sinus?
			com temp				; zweite Hlfte: horizontal Spiegeln

	K1_freq_X:
			sts Kanal1_Tonwert, temp


	; Frequenzgenerator Kanal 2 kann mit Kanal 3 synchronisiert oder Ringmoduliert werden
	; Kanal 3 muss nur laufen, wenn aktiv, sonst egal, da eh nicht hrbar
	; mglich wre bspw. Wellenform
	;						Wellenform = 0bis4 fr Ringmod
	;						Wellenform = 5 fr sync? eigentliche Wellenform egal
	;						Wellenform = 6 fr aus



			lds ZL, Kanal3_Wellenform
			cpi ZL, 0x06			; Kanal aus??
			brne Kanal3_los
			rjmp Kanal2_freqGen


	Kanal3_los:
			; notwendige Werte aus dem SRAM holen zur Bearbeitung
			lds temp, Kanal3_Frequenz_L
			lds temp2, Kanal3_Frequenz_H
			lds CounterTemp1, K3_CL
			lds CounterTemp2, K3_CM
			lds CounterTemp3, K3_CH

			; Frequenzwerte den aktuellen Tons addieren
			; es werden nur 20 bit verwendet
			add CounterTemp1, temp
			adc CounterTemp2, temp2
			adc CounterTemp3, zero
			brhc K3_FreqGen_1
			clr CounterTemp3
	K3_FreqGen_1:
			cpi ZL, 0x05				; sync? dann Kanal2 auf 0
			brne K3_FreqGen_2
			sts K2_CL, zero
			sts K2_CM, zero
			sts K2_CH, zero

			sts K3_CH, CounterTemp3
			sts K3_CM, CounterTemp2
			sts K3_CL, CounterTemp1

			clr temp
			rjmp Kanal2_freq_sync		; weiter mit Kanal2, Wellenform ist egal bei sync

		; ab hier Ringmod mit Kanal2 -> Wellenform erzeugen
	K3_FreqGen_2:
			; die neuen Werte wieder im SRAM sichern
			sts K3_CH, CounterTemp3
			sts K3_CM, CounterTemp2
			sts K3_CL, CounterTemp1

			; die oberen 8 bit (von20) weiterverwenden
			mov temp, CounterTemp2
			mov temp2, CounterTemp3
			swap temp
			andi temp, 0b00001111
			swap temp2
			or temp, temp2

			; verzweigen nach Wellenform
			mov temp2, ZL
			ldi ZL, low (K3_freq_Sprungtab)
			ldi ZH, high(K3_freq_Sprungtab)
			add ZL, temp2
			adc ZH, zero
			ijmp

	K3_freq_Sprungtab:
			rjmp Kanal3_Rechteck
			rjmp K3_freq_X
			rjmp Kanal3_Dreieck
			rjmp Kanal3_Rauschen
			rjmp Kanal3_Sinus

	Kanal3_Rechteck:
			; bei Rechteck
			lds temp2, Kanal3_Pulsweite
			cp temp, temp2
			brsh K3_freq_1
			clr temp
			rjmp K3_freq_X
		K3_freq_1:
			ser temp
			rjmp K3_freq_X
	
	Kanal3_Dreieck:
			; bei Dreieck
			; Tonwert = 2*temp fr temp<0x7F sonst 0xFF-(2*temp)
			lsl temp
			brcs K3_freq_2
			rjmp K3_freq_X
		K3_freq_2:
			com temp
			rjmp K3_freq_X

	Kanal3_Rauschen:
			ldi ZL, low (rauschen8bit*2)
			ldi ZH, high(rauschen8bit*2)
			swap CounterTemp1
			add temp, CounterTemp1

#if defined(m_Noise_Short)
			rjmp K3_sinus1
#else
			add ZL, temp
			adc ZH, zero
			lpm temp, Z
			rjmp K3_freq_X
#endif


	Kanal3_Sinus:
			ldi ZL, low (sin8bit*2)
			ldi ZH, high(sin8bit*2)

	K3_sinus1:
			mov temp2, temp
			andi temp, 0b00111111

			sbrs temp2, 6
			rjmp K3_sinus2

			ldi temp3, 0x3F
			sub temp3, temp
			mov temp, temp3

	K3_sinus2:
			add ZL, temp
			adc ZH, zero

			lpm temp, Z

			sbrs temp2, 7
			com temp

	K3_freq_X:
			sts Kanal3_Tonwert, temp




	; Frequenzgenerator Kanal 2


	Kanal2_freqGen:
			; notwendige Werte aus dem SRAM holen zur Bearbeitung
			lds temp, Kanal2_Frequenz_L
			lds temp2, Kanal2_Frequenz_H
			lds CounterTemp1, K2_CL
			lds CounterTemp2, K2_CM
			lds CounterTemp3, K2_CH

			; Frequenzwerte den aktuellen Tons addieren
			; es werden nur 20 bit verwendet
			add CounterTemp1, temp
			adc CounterTemp2, temp2
			adc CounterTemp3, zero
			brhc K2_FreqGen_1
			clr CounterTemp3

	K2_FreqGen_1:
			; die neuen Werte wieder im SRAM sichern
			sts K2_CH, CounterTemp3
			sts K2_CM, CounterTemp2
			sts K2_CL, CounterTemp1

			; die oberen 8 bit weiterverwenden
			mov temp, CounterTemp2
			mov temp2, CounterTemp3
			swap temp
			andi temp, 0b00001111
			swap temp2
			or temp, temp2

	Kanal2_freq_sync:

			; verzweigen nach Wellenform
			ldi ZL, low (K2_freq_Sprungtab)
			ldi ZH, high(K2_freq_Sprungtab)
			lds temp2, Kanal2_Wellenform
			add ZL, temp2
			adc ZH, zero
			ijmp


	K2_freq_Sprungtab:
			rjmp Kanal2_Rechteck
			rjmp K2_freq_X
			rjmp Kanal2_Dreieck
			rjmp Kanal2_Rauschen
			rjmp Kanal2_Sinus

	Kanal2_Rechteck:
			; bei Rechteck
			lds temp2, Kanal2_Pulsweite
			cp temp, temp2
			brsh K2_freq_1
			clr temp
			rjmp K2_freq_X
		K2_freq_1:
			ser temp
			rjmp K2_freq_X

	Kanal2_Dreieck:
			; bei Dreieck
			lsl temp
			brcs K2_freq_2
			rjmp K2_freq_X
		K2_freq_2:
			com temp
			rjmp K2_freq_X

	Kanal2_Rauschen:
			ldi ZL, low (rauschen8bit*2)
			ldi ZH, high(rauschen8bit*2)
			swap CounterTemp1
			add temp, CounterTemp1

#if defined(m_Noise_Short)
			rjmp K2_sinus1
#else
			add ZL, temp
			adc ZH, zero
			lpm temp, Z
			rjmp K2_freq_X
#endif


	Kanal2_Sinus:
			ldi ZL, low (sin8bit*2)
			ldi ZH, high(sin8bit*2)

	K2_sinus1:
			mov temp2, temp
			andi temp, 0b00111111

			sbrs temp2, 6
			rjmp K2_sinus2

			ldi temp3, 0x3F
			sub temp3, temp
			mov temp, temp3

	K2_sinus2:


			add ZL, temp
			adc ZH, zero

			lpm temp, Z

			sbrs temp2, 7
			com temp

		K2_freq_X:
			; Tonwert in temp
			sts Kanal2_Tonwert, temp



			; noch die Ringmodulation Kanal2+3 einfgen
			lds temp2, Kanal3_Wellenform
			cpi temp2, 0x05
			brsh K1_huellgen		; verzweigen, wenn keine RingModulation

			lds temp2, Kanal3_Tonwert

			clr mult_R				; Ergebnisregister lschen

			; einfache Multiplikation der Tonwerte
			lsr temp		
			sbrc temp2,7
			adc mult_R,temp

			lsr temp	
			sbrc temp2,6
			adc mult_R,temp

			lsr temp	
			sbrc temp2,5
			adc mult_R,temp

			lsr temp
			sbrc temp2,4
			adc mult_R,temp

			lsr temp	
			sbrc temp2,3
			adc mult_R,temp

			lsr temp
			sbrc temp2,2
			adc mult_R,temp

			lsr temp	
			sbrc temp2,1
			adc mult_R,temp

			lsr temp
			sbrc temp2,0
			adc mult_R,temp

			sts Kanal2_Tonwert, mult_R


; *************************************************************************
; Huellkurvengeneratoren, grundlegender Aufbau:
;	Es gibt 3 Zhler, einen(TxDauerM) ber volle 256 Aufrufe, der die
;	jeweilige Kurve steuert, sowie zwei geschachtelte Zhler fr die
;	Gesamtdauer.
;	Zuerst wird geprft, ob der Zustand und die nderung zum letzten
;	Zustand des Gate-bits geprft und entsprechend verzweigt
;	


; ab hier Hllkurvengenerator fr Kanal1
	K1_huellgen:
			ldi temp, 0b00000001
			ldi temp2, 0b00000001
			and temp, GateAkt
			and temp2, GateAlt
			eor temp, temp2				; Wechsel im Gate-bit fr Kanal 1?
			breq keinwechselK1
			sbrs temp2, 0				; skip wenn vorher eins
			rjmp wechselK1auf1
		; Gate von 1 auf 0 -> Release auslsen
			lds T1DauerL, Kanal1_Release		; Zhler initialisieren
			inc T1DauerL
			mov T1DauerH, T1DauerL
			clr T1DauerM

			lds temp, Ton1_huellwert
			sts Ton1_R2, temp
			rjmp Kana1_huellende
		
		;wechsel von 0 auf 1: Ton auslsen, Hllkurve starten
	WechselK1auf1:
			lds temp, Ton1_huellwert	; Aktuellen Wert der Huellkurve speichern
										; fr weichen bergang zwischen zwei Tnen
			sbrs GateAkt, 0x06
			clr temp
			sts Ton1_A2, temp

			lds T1DauerL, Kanal1_Attac		; Zhler initialisieren
			inc T1DauerL
			mov T1DauerH, T1DauerL
			clr T1DauerM

			ldi ZH, high(ton1_attac)	; Zum schnellen Verzweigen, Adresse speichern
			sts ton1_ADS_addrH, ZH
			ldi ZL, low (ton1_attac)
			sts ton1_ADS_addrL, ZL
			ijmp

		;normal weiter (kein Wechsel im Gate)
	keinwechselK1:
			sbrc temp2, 0				; Skip, wenn Gate 0
			rjmp Kanal1_ADS				;10 bei Sprung in die norm. Hllkurve


			lds temp, Ton1_huellwert	; Ist der Huellwert schon auf 0?
			tst temp
			brne ton1_release			; Sonst Release zu Ende fhren
			rjmp Kana1_huellende

	ton1_release:
			dec T1DauerL				; 'innere' Schleife
			breq ton1_release_0
			rjmp Kana1_huellende
	ton1_release_0:
			dec T1DauerH				; 'uere' Schleife
			breq Ton1_release_1

			lds T1DauerL, Kanal1_Release
			inc T1DauerL

			rjmp Kana1_huellende

	ton1_release_1:	
			dec T1DauerM				
			breq Ton1_release_2
										; Huellwert ndert sich
			lds mult_1, Ton1_R2
			mov mult_2, T1DauerM
			rcall mult_m2				; Startwert * Dauer^2

			sts Ton1_huellwert, mult_R

			lds T1DauerL, Kanal1_Release
			inc T1DauerL
			mov T1DauerH, T1DauerL

			rjmp Kana1_huellende
	ton1_release_2:
			sts ton1_huellwert, zero
			rjmp Kana1_huellende


;###########################################################

Kanal1_ADS:
			dec T1DauerL
			breq ton1_neu1
			rjmp Kana1_huellende			;		 keine nderung in der Hllkurve
	Ton1_neu1:
			lds ZH, ton1_ADS_addrH			; schneller Sprung zu Attac, Decay oder Sustain
			lds ZL, ton1_ADS_addrL

			ijmp

	ton1_attac:
			dec T1DauerH
			breq ton1_attac_1

			lds T1DauerL, Kanal1_Attac
			inc T1DauerL

			rjmp Kana1_huellende
	ton1_attac_1:
			dec T1DauerM
			breq ton1_attac_2

			lds temp, Ton1_A2				; Startwert holen
			com temp						; und umkehren
			mov mult_2, T1DauerM
			com mult_2

			mov mult_1, temp
			rcall mult_m1					; Startwert * Dauer

			com temp
			add temp, mult_R

			sts Ton1_huellwert, temp

			lds T1DauerL, Kanal1_Attac
			inc T1DauerL
			mov T1DauerH, T1DauerL

			rjmp Kana1_huellende

	ton1_attac_2:
			lds T1DauerL, Kanal1_Decay
			inc T1DauerL
			mov T1DauerH, T1DauerL

			ser temp
			sts Ton1_huellwert, temp		; zur Sicherheit
			ldi temp, high(ton1_decay)		; umstellen auf Decay-Phase
			sts ton1_ADS_addrH, temp
			ldi temp, low (ton1_decay)
			sts ton1_ADS_addrL, temp
			rjmp Kana1_huellende


	ton1_decay:
			dec T1DauerH
			breq ton1_decay_1

			lds T1DauerL, Kanal1_Decay
			inc T1DauerL

			rjmp Kana1_huellende
	ton1_decay_1:
			dec T1DauerM
			breq ton1_decay_2

			lds temp2, Kanal1_Sustain			; Zielwert holen
			com temp2							;

			mov mult_1, temp2
			mov mult_2, T1DauerM
			rcall mult_m2						; sustain * dauer^2

			com temp2							; wieder Sustainwert herstellen
			add mult_R, temp2
			sts Ton1_huellwert, mult_R

			lds T1DauerL, Kanal1_Decay
			inc T1DauerL
			mov T1DauerH, T1DauerL

			rjmp Kana1_huellende

	ton1_decay_2:
			lds temp, Kanal1_Sustain
			sts Ton1_huellwert, temp			; zur Sicherheit, falls vorher nicht erreicht
			ldi temp, 0x40
			mov T1DauerL, temp
			ldi temp, high(ton1_sustain)		; umstellen auf Sustain-Phase
			sts ton1_ADS_addrH, temp
			ldi temp, low (ton1_sustain)
			sts ton1_ADS_addrL, temp
			rjmp Kana1_huellende


	ton1_sustain:
			; Auslsen der Release-Phase ber Gate
			ldi temp, 0x40
			mov T1DauerL, temp				; nur die Auflsung

Kana1_huellende:
			; Endemarkierung, ab hier kann Kanal2 laufen


; ab hier Hllkurvengenerator fr Kanal2

			ldi temp, 0b00000010
			ldi temp2, 0b00000010
			and temp, GateAkt
			and temp2, GateAlt
			mov GateAlt, GateAkt	; hier richtig, auer bei drittem Kanal
			eor temp, temp2
			breq keinwechselK2
			sbrs temp2, 1			; skip wenn vorher eins
			rjmp wechselK2auf1
		; Gate von 1 auf 0 -> Release auslsen
			lds T2DauerL, Kanal2_Release
			inc T2DauerL
			mov T2DauerH, T2DauerL

			clr T2DauerM
			lds temp, Ton2_huellwert
			sts Ton2_R2, temp
			rjmp Kanal2_huellende
		
		;wechsel von 0 auf 1: Ton auslsen, Hllkurve starten
	WechselK2auf1:
			ser temp
			mov T2DauerM, temp
			lds temp, Ton2_huellwert

			sbrs GateAkt, 0x07				; weicher bergang?
			clr temp

			sts Ton2_A2, temp

			lds T2DauerL, Kanal2_Attac
			inc T2DauerL
			mov T2DauerH, T2DauerL
			ldi ZH, high(ton2_attac)
			sts ton2_ADS_addrH, ZH
			ldi ZL, low (ton2_attac)
			sts ton2_ADS_addrL, ZL
			ijmp

		;normal weiter (kein Wechsel im Gate)
	keinwechselK2:
			sbrc temp2, 1					; Skip, wenn Gate 0
			rjmp Kanal2_ADS

			lds temp, Ton2_huellwert
			tst temp
			brne ton2_release
			rjmp Kanal2_huellende

	ton2_release:
			dec T2DauerL
			breq ton2_release_0
			rjmp Kanal2_huellende
	ton2_release_0:
			dec T2DauerH
			breq Ton2_release_1
			lds T2DauerL, Kanal2_Release
			inc T2DauerL
			rjmp Kanal2_huellende

	ton2_release_1:
			dec T2DauerM			
			breq Ton2_release_2

			lds mult_1, Ton2_R2
			mov mult_2, T2DauerM
			rcall mult_m2

			sts Ton2_huellwert, mult_R

			lds T2DauerL, Kanal2_Release
			inc T2DauerL
			mov T2DauerH, T2DauerL

			rjmp Kanal2_huellende
	ton2_release_2:
			sts ton2_huellwert, zero
			rjmp Kanal2_huellende

;###########################################################

Kanal2_ADS:
			dec T2DauerL
			breq ton2_neu1
			rjmp Kanal2_huellende			;		 keine nderung in der Hllkurve
	Ton2_neu1:
			lds ZH, ton2_ADS_addrH
			lds ZL, ton2_ADS_addrL

			ijmp

	ton2_attac:
			dec T2DauerH
			breq ton2_attac_1

			lds T2DauerL, Kanal2_Attac
			inc T2DauerL

			rjmp Kanal2_huellende
	ton2_attac_1:
			dec T2DauerM
			breq ton2_attac_2

			lds temp, Ton2_A2
			com temp
			mov mult_2, T2DauerM
			com mult_2

			mov mult_1, temp
			rcall mult_m1
			com temp
			add temp, mult_R

			sts Ton2_huellwert, temp

			lds T2DauerL, Kanal2_Attac
			inc T2DauerL
			mov T2DauerH, T2DauerL

			rjmp Kanal2_huellende

	ton2_attac_2:
			lds T2DauerL, Kanal2_Decay
			inc T2DauerL
			mov T2DauerH, T2DauerL

			ser temp
			sts Ton2_huellwert, temp			; zur Sicherheit
			ldi temp, high(ton2_decay)
			sts ton2_ADS_addrH, temp
			ldi temp, low (ton2_decay)
			sts ton2_ADS_addrL, temp
			rjmp Kanal2_huellende

	ton2_decay:
			dec T2DauerH
			breq ton2_decay_1

			lds T2DauerL, Kanal2_Decay
			inc T2DauerL

			rjmp Kanal2_huellende
	ton2_decay_1:
			dec T2DauerM
			breq ton2_decay_2

			lds temp2, Kanal2_Sustain
			com temp2							; 0xFF-temp2

			mov mult_1, temp2
			mov mult_2, T2DauerM
			rcall mult_m2

			com temp2							; wieder Sustainwert herstellen
			add mult_R, temp2
			sts Ton2_huellwert, mult_R

			lds T2DauerL, Kanal2_Decay
			inc T2DauerL
			mov T2DauerH, T2DauerL
			rjmp Kanal2_huellende

	ton2_decay_2:
			lds temp, Kanal2_Sustain
			sts Ton2_huellwert, temp			; zur Sicherheit, falls vorher nicht erreicht
			ldi temp, 0x40
			mov T2DauerL, temp
			ldi temp, high(ton2_sustain)
			sts ton2_ADS_addrH, temp
			ldi temp, low (ton2_sustain)
			sts ton2_ADS_addrL, temp
			rjmp Kanal2_huellende

	ton2_sustain:
			ldi temp, 0x40
			mov T2DauerL, temp				; nur die Auflsung

Kanal2_huellende:
			; Endemarkierung, ab hier kann Kanal3 laufen



	ton2_weiter_2:
			lds temp2, Ton2_huellwert
			lds temp, Kanal2_tonwert





			; im Folgenden wird der Tonwert als Signed aufgefasst und mit dem Huellwert
			; multipliziert

			subi temp, 0x80			; Vorzeichen drehen
; ******************************
;		out OCR1A, temp
; ******************************

			clr mult_R				; Ergebnisregister lschen

			bst temp, 7				; Vorzeichen merken

			lsr temp2
			sbrc temp,7
			com temp


			lsr temp2
			sbrc temp,6
			adc mult_R, temp2

			lsr temp2
			sbrc temp,5
			adc mult_R, temp2

			lsr temp2
			sbrc temp,4
			adc mult_R, temp2

			lsr temp2
			sbrc temp,3
			adc mult_R, temp2

			lsr temp2
			sbrc temp,2
			adc mult_R, temp2

			lsr temp2
			sbrc temp,1
			adc mult_R, temp2

			lsr temp2
			sbrc temp,0
			adc mult_R, temp2

			brtc ton2_weiter_3			; Vorzeichen wieder herstellen

			com mult_R
			inc mult_R

		ton2_weiter_3:
			mov temp, mult_R
			subi temp, 0x80				; und wieder nach unsigned wandeln


	Kanal1_Modulator:
	ton1_weiter_2:
			lds temp2, Ton1_huellwert
			lds temp3, Kanal1_tonwert
			subi temp3, 0x80			; Vorzeichen drehen

			clr mult_R					; Ergebnisregister lschen

			bst temp3, 7

			lsr temp2
			sbrc temp3,7
			com temp3

			lsr temp2
			sbrc temp3,6
			adc mult_R,temp2

			lsr temp2
			sbrc temp3,5
			adc mult_R,temp2

			lsr temp2
			sbrc temp3,4
			adc mult_R,temp2

			lsr temp2
			sbrc temp3,3
			adc mult_R,temp2

			lsr temp2
			sbrc temp3,2
			adc mult_R,temp2

			lsr temp2
			sbrc temp3,1
			adc mult_R,temp2

			lsr temp2
			sbrc temp3,0
			adc mult_R,temp2

			brts ton1_weiter_3

			com mult_R
			inc mult_R

	ton1_weiter_3:
			mov temp3, mult_R
			subi temp3, 0x80			; 14

	mischer:
			; da sich in temp und temp3 8-bit-werte befinden, mssen diese gekrzt werden

		; unterschiedliche Versionen:
		;	beide Kanle addieren, je nach timer auf 8- oder 9-bit

#if defined(m_Tiny85) | defined(m_Tiny25) | defined(m_Tiny45)
			add temp, temp3
			ror temp
			out OCR1A, temp
#elif defined(m_Tiny2313)
			add temp, temp3
			adc temp2, zero
			out OCR1AH, temp2
			out OCR1AL, temp
#else
#error "Kein passender Prozessor ausgewhlt!!!"
#endif



	timer0ovf_ende:
			; Register wiederherstellen und raus aus der ISR
			pop mult_2
			pop mult_1
			pop mult_R
			pop ZH
			pop ZL
			pop temp
			out SREG, temp
			reti		;17


; Takte gesamt maximal:
;	 13 ISR Anfang
;	192 Freq 1-3
;	180 Hllk 1+2
;	 74 Mischer und Ausgabe
;	 17 ISR Ende
;  =====
;	476 Takte maximal (bisher)
;	+18 neuer Sinus = 494 gesamt + Einsprung



mult_m1:
		; Multiplikation von zwei 8-bit-werten zu 8-bit
		; mathematisch nicht ganz korrekt, aber schnell und fr den Zweck ausreichend
		clr mult_R

		lsr mult_1		
		sbrc mult_2,7
		adc mult_R,mult_1

		lsr mult_1	
		sbrc mult_2,6
		adc mult_R,mult_1

		lsr mult_1	
		sbrc mult_2,5
		adc mult_R,mult_1

		lsr mult_1
		sbrc mult_2,4
		adc mult_R,mult_1

		lsr mult_1	
		sbrc mult_2,3
		adc mult_R,mult_1

		lsr mult_1
		sbrc mult_2,2
		adc mult_R,mult_1

		lsr mult_1	
		sbrc mult_2,1
		adc mult_R,mult_1

		lsr mult_1
		sbrc mult_2,0
		adc mult_R,mult_1

		ret


mult_m2:
		; hnlich wie zuvor, nur R= A x A x B damit bei decay und release eine
		; leichte Kurve ensteht statt einer Geraden
		clr mult_R

		lsr mult_1		
		sbrc mult_2,7
		adc mult_R,mult_1

		lsr mult_1	
		sbrc mult_2,6
		adc mult_R,mult_1

		lsr mult_1	
		sbrc mult_2,5
		adc mult_R,mult_1

		lsr mult_1
		sbrc mult_2,4
		adc mult_R,mult_1

		lsr mult_1	
		sbrc mult_2,3
		adc mult_R,mult_1

		lsr mult_1
		sbrc mult_2,2
		adc mult_R,mult_1

		lsr mult_1	
		sbrc mult_2,1
		adc mult_R,mult_1

		lsr mult_1
		sbrc mult_2,0
		adc mult_R,mult_1



		lsr mult_R		
		sbrc mult_2,7
		adc mult_1,mult_R

		lsr mult_R	
		sbrc mult_2,6
		adc mult_1,mult_R

		lsr mult_R	
		sbrc mult_2,5
		adc mult_1,mult_R

		lsr mult_R
		sbrc mult_2,4
		adc mult_1,mult_R

		lsr mult_R	
		sbrc mult_2,3
		adc mult_1,mult_R

		lsr mult_R
		sbrc mult_2,2
		adc mult_1,mult_R

		lsr mult_R	
		sbrc mult_2,1
		adc mult_1,mult_R

		lsr mult_R
		sbrc mult_2,0
		adc mult_1,mult_R

		mov mult_R, mult_1

		ret



; ################################################################################################
; ################################################################################################
; ##			           								 									    ##
; ##									  Initialisierung		  								##
; ##			            									  								##
; ################################################################################################
; ################################################################################################



reset:
			; Stackpointer initialisieren, SPL nur bei mehr als 128 Bytes SRAM
#if defined(m_Tiny85) | defined(m_Tiny45)
			ldi temp, high(RAMEND)
			out SPH, temp
#endif
			ldi temp, low (RAMEND)
			out SPL, temp


			; Timer initialisieren, 8-bit mit Prescaler 2 bei Tiny85
			;	9-bit ohne Prescaler bei Tiny2313
#if defined(m_Tiny85) | defined(m_Tiny25) | defined(m_Tiny45)
		; Timer 1 ist fr die Tonausgabe zustndig, Register entsprechend setzen
			ldi temp, 1<<PWM1A | 1<<COM1A1 | 1<<CS11
			out TCCR1, temp
			ser temp
			out OCR1C, temp
			; PORT-Register setzen, OCR1A als Ausgang
			sbi DDRB, PB1


#elif defined(m_Tiny2313)
		; Tiny2313-Version: 9-bit Fast-PWM no Presc.
			ldi temp, 1<<COM1A1 | 1<<WGM11
			out TCCR1A, temp
			ldi temp, 1<<WGM12 | 1<<CS10
			out TCCR1B, temp
			; OC1A ist PB3 auf Tiny2313
			sbi DDRB, PB3
#else
#error "Kein Prozessor ausgewhlt!!!"
#endif


			ldi temp, 1<<TOIE1				; Timer 1 overflowinterrupt aktivieren
			out TIMSK, temp

			; Null-Register setzen
			clr zero

			; Zaehler etc. reseten
			clr CounterTemp1
			clr CounterTemp2
			clr CounterTemp3

			sts K1_CL, zero
			sts K1_CM, zero
			sts K1_CH, zero

			sts K2_CL, zero
			sts K2_CM, zero
			sts K2_CH, zero

			sts ton1_huellwert, zero
			sts ton2_huellwert, zero

			clr GateAlt


			sei										; damit's nicht vergessen wird


; 'Hauptprogramm' einbinden:
.include "MAIN_inc.asm"

; Sinus und Rausch-Tabellen einbinden
.include "FreqGenTables_inc.asm"

; SRAM_Deklarationen einbinden
.include "SRAM_inc.asm"


